# -*- coding: utf-8 -*-

import pandas as pd
from beartype.typing import List
from dramkit.iotools import load_df_pd


def load_score_system(excel_path: str) -> List[dict]:
    '''
    | 从Excel文件中读取评价指标体系
    | excel中列名顺序必须为：
    |   评价目标、一级指标、一级权重、二级指标、二级权重、三级指标、三级权重、···
    '''
    df = load_df_pd(excel_path)
    df = df.fillna(method='ffill')
    dfcols = list(df.columns)
    def _gen_level(cols3):
        assert len(cols3) == 3
        res = {}
        parent = df[cols3[0]].unique()
        for p in parent:
            tmp = df[df[cols3[0]] == p][cols3[1:]]
            tmp[cols3[2]] = tmp[cols3[2]].astype(float)
            res[p] = tmp.set_index(cols3[1])[cols3[2]].to_dict()
        return res
    score_sys = []
    cols3 = dfcols[:3]
    score_sys.append(_gen_level(cols3))
    for k in range(1, len(dfcols)-2, 2):
        cols3 = [dfcols[k]] + dfcols[k+2:k+4]
        score_sys.append(_gen_level(cols3))
    return score_sys


def cal_score(df: pd.DataFrame, score_sys: list) -> pd.DataFrame:
    '''score_sys必须符合 :py:func:`load_score_system` 函数的输出'''
    inds = link_lists([list(x.keys()) for x in score_sys[-1].values()])
    assert all([x in df.columns for x in inds])
    cols_ = []
    res = df.copy()
    for level in score_sys[::-1]:
        for p, chlds in level.items():
            cols_.append(p)
            res[p] = eval('+'.join(['res["%s"]*%s'%(k, v) for k, v in chlds.items()]))
    # res = res[cols_[::-1] + list(df.columns)]
    res = res[cols_[::-1]]
    return res
    
if __name__ == '__main__':
    import numpy as np
    from dramkit.gentools import link_lists, TimeRecoder
    tr = TimeRecoder()
    
    excel_path = './_test/level_score_system.xlsx'
    score_sys = load_score_system(excel_path)
    
    inds = link_lists([list(x.keys()) for x in score_sys[-1].values()])
    n = 1000
    df = pd.DataFrame(columns=inds,
                      index=['smp_%s'%k for k in range(1, n+1)])
    for ind in inds+['a', 'b', 'c']:
        df[ind] = np.random.rand(df.shape[0])
    df_score = cal_score(df, score_sys)
    
    
    tr.used()
    
    
    
    
    
    
    
    
    
    
    